#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <time.h>

// #define TEST_TIME
static long waiters = 0;
static int done = 0;
struct CRITICAL_SECTION* lock;

struct CRITICAL_SECTION{
    pthread_mutex_t mutex;
	int i;
};

void Inc_waiters()
{
    ++waiters;
}

long Dec_waiters()
{
    return --waiters;
}

void Enter(struct CRITICAL_SECTION* lock)
{
	lock->i = 1;
	pthread_mutex_lock(&lock->mutex); 
}

void Exit(struct CRITICAL_SECTION* lock)
{
    pthread_mutex_unlock(&lock->mutex);
}

void * once(void* arg)
{        
    static int done = 0;

    if(done)
        return 0;

    Inc_waiters();
    Enter(lock);
    printf("Enter\n");

    if(!done)
    {
        done = 1;
    }

    printf("Exit\n");

    Exit(lock);
	
    if(!Dec_waiters())
    {
        printf("Decrement waiters\n");
        free(lock);
		printf("free lock\n");
        lock = NULL;
    }
	return NULL;
}

#define thread_size 2

int main()
{
    lock = (struct CRITICAL_SECTION*)malloc(sizeof(struct CRITICAL_SECTION));
	pthread_mutex_init(&lock->mutex, NULL);
#ifdef TEST_TIME
    static double run_time_begin;
    static double run_time_end;
    static double run_time_total;
    run_time_begin = clock();
#endif
    pthread_t t[thread_size];

    for(int i = 0; i < thread_size;)
    {
        pthread_create(&t[i], NULL, once, NULL);
        pthread_create(&t[i+1], NULL, once, NULL);
        i += 2;
    }

    struct timeval start, end;
    gettimeofday( &start, NULL );

    for(int i = 0; i < thread_size;)
    {
        pthread_join(t[i+1], NULL);
		pthread_join(t[i], NULL);
        i += 2;
    }

    gettimeofday( &end, NULL );
    int timeuse = 1000000 * ( end.tv_sec - start.tv_sec ) + end.tv_usec -start.tv_usec;
    //printf("time: %d us\n", timeuse);

    printf("\nprogram-successful-exit\n");
#ifdef TEST_TIME
    run_time_end = clock();
    run_time_total = run_time_end - run_time_begin;
    printf("test-the-total-time: %.3lf\n", (double)(run_time_total/CLOCKS_PER_SEC)*1000);
#endif
    return 0;
}
