# Test

Please try to perform following command:

```sh
# setup the environment variables in the root directory of the tool
$ source tool/init_env.sh

# compile the program and get bit code
$ cd $ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265
$ ./cleanDIR.sh
$ clang -g -emit-llvm -c ./2017-15265.cpp -o 2017-15265.bc

# perform static analysis
$ $ROOT_DIR/tool/staticAnalysis/staticAnalysis.sh 2017-15265

# complie the instrumented program with ASAN
$ export Con_PATH=$ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/ConConfig.2017-15265
$ $ROOT_DIR/tool/staticAnalysis/DBDS-INSTRU/dbds-clang-fast++ -g -fsanitize=address ./2017-15265.cpp -o 2017-15265 -lpthread -ldl

# perform DBDS
$ $ROOT_DIR/tool/DBDS/run_PDS.py -d 2 ./2017-15265
```

Then you will get the results.

```sh
Start Testing!
test 0001
test 0002
...
```

The ASAN output for use-after-free Bug:
```sh
=================================================================
==70209==ERROR: AddressSanitizer: heap-use-after-free on address 0x607000000038 at pc 0x0000004c9c48 bp 0x7f5363b19e90 sp 0x7f5363b19e88
WRITE of size 4 at 0x607000000038 thread T1
    #0 0x4c9c47  (/ConcurrencyFuzzer/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/2017-15265+0x4c9c47)
    #1 0x4c9a39  (/ConcurrencyFuzzer/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/2017-15265+0x4c9a39)
    #2 0x7f53676eb6b9  (/lib/x86_64-linux-gnu/libpthread.so.0+0x76b9)
    #3 0x7f53667744dc  (/lib/x86_64-linux-gnu/libc.so.6+0x1074dc)

0x607000000038 is located 24 bytes inside of 72-byte region [0x607000000020,0x607000000068)
freed by thread T2 here:
    #0 0x4947ed  (/ConcurrencyFuzzer/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/2017-15265+0x4947ed)
    #1 0x4c9831  (/ConcurrencyFuzzer/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/2017-15265+0x4c9831)

previously allocated by thread T1 here:
    #0 0x495182  (/ConcurrencyFuzzer/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/2017-15265+0x495182)
    #1 0x4c90c6  (/ConcurrencyFuzzer/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/2017-15265+0x4c90c6)

Thread T1 created by T0 here:
    #0 0x47f1fa  (/ConcurrencyFuzzer/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/2017-15265+0x47f1fa)
    #1 0x4c9fc3  (/ConcurrencyFuzzer/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/2017-15265+0x4c9fc3)
    #2 0x7f536668d83f  (/lib/x86_64-linux-gnu/libc.so.6+0x2083f)

Thread T2 created by T0 here:
    #0 0x47f1fa  (/ConcurrencyFuzzer/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/2017-15265+0x47f1fa)
    #1 0x4c9fe8  (/ConcurrencyFuzzer/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/2017-15265+0x4c9fe8)
    #2 0x7f536668d83f  (/lib/x86_64-linux-gnu/libc.so.6+0x2083f)

SUMMARY: AddressSanitizer: heap-use-after-free (/ConcurrencyFuzzer/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/2017-15265+0x4c9c47)
Shadow bytes around the buggy address:
  0x0c0e7fff7fb0: 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
  0x0c0e7fff7fc0: 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
  0x0c0e7fff7fd0: 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
  0x0c0e7fff7fe0: 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
  0x0c0e7fff7ff0: 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
=>0x0c0e7fff8000: fa fa fa fa fd fd fd[fd]fd fd fd fd fd fa fa fa
  0x0c0e7fff8010: fa fa fa fa fa fa fa fa fa fa fa fa fa fa fa fa
  0x0c0e7fff8020: fa fa fa fa fa fa fa fa fa fa fa fa fa fa fa fa
  0x0c0e7fff8030: fa fa fa fa fa fa fa fa fa fa fa fa fa fa fa fa
  0x0c0e7fff8040: fa fa fa fa fa fa fa fa fa fa fa fa fa fa fa fa
  0x0c0e7fff8050: fa fa fa fa fa fa fa fa fa fa fa fa fa fa fa fa
Shadow byte legend (one shadow byte represents 8 application bytes):
  Addressable:           00
  Partially addressable: 01 02 03 04 05 06 07
  Heap left redzone:       fa
  Freed heap region:       fd
  Stack left redzone:      f1
  Stack mid redzone:       f2
  Stack right redzone:     f3
  Stack after return:      f5
  Stack use after scope:   f8
  Global redzone:          f9
  Global init order:       f6
  Poisoned by user:        f7
  Container overflow:      fc
  Array cookie:            ac
  Intra object redzone:    bb
  ASan internal:           fe
  Left alloca redzone:     ca
  Right alloca redzone:    cb
  Shadow gap:              cc
==70209==ABORTING
```

Use `addr2line -e ./2017-15265 0x4c9c47` and `addr2line -e ./2017-15265 0x4c9a39` to see the debug info
```
/ConcurrencyFuzzer/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/./2017-15265.cpp:105
/ConcurrencyFuzzer/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/./2017-15265.cpp:214
```
