#!/bin/bash

# For Mac
if [ $(command uname) == "Darwin" ]; then
	if ! [ -x "$(command -v greadlink)"  ]; then
		brew install coreutils
	fi
	BIN_PATH=$(greadlink -f "$0")
	ROOT_DIR=$(dirname $(dirname $(dirname $BIN_PATH)))
# For Linux
else
	BIN_PATH=$(readlink -f "$0")
	ROOT_DIR=$(dirname $(dirname $(dirname $BIN_PATH)))
fi

export ROOT_DIR=${ROOT_DIR}
export PATH=${ROOT_DIR}/clang+llvm/bin:${ROOT_DIR}/tool/SVF/Release-build/bin:$PATH
export LD_LIBRARY_PATH=${ROOT_DIR}/clang+llvm/lib${LD_LIBRARY_PATH:+:$LD_LIBRARY_PATH}


${ROOT_DIR}/tool/staticAnalysis/clean_shm.py > /dev/null
echo "Installation completed. Everything's fine!"

set -eux

# CVE-2009-3547
cd $ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2009-3547
./cleanDIR.sh
clang++ -g -emit-llvm -c ./2009-3547.cpp -o 2009-3547.bc
$ROOT_DIR/tool/staticAnalysis/staticAnalysis.sh 2009-3547
export Con_PATH=$ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2009-3547/ConConfig.2009-3547
$ROOT_DIR/tool/staticAnalysis/DBDS-INSTRU/dbds-clang-fast++ -g -fsanitize=address ./2009-3547.cpp -o 2009-3547 -lpthread -ldl

# CVE-2011-2183
cd $ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2011-2183
./cleanDIR.sh
clang++ -g -emit-llvm -c ./2011-2183.cpp -o 2011-2183.bc
$ROOT_DIR/tool/staticAnalysis/staticAnalysis.sh 2011-2183
export Con_PATH=$ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2011-2183/ConConfig.2011-2183
$ROOT_DIR/tool/staticAnalysis/DBDS-INSTRU/dbds-clang-fast++ -g -fsanitize=address ./2011-2183.cpp -o 2011-2183 -lpthread -ldl

# CVE-2013-1792
cd $ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2013-1792
./cleanDIR.sh
clang++ -g -emit-llvm -c ./2013-1792.cpp -o 2013-1792.bc
$ROOT_DIR/tool/staticAnalysis/staticAnalysis.sh 2013-1792
export Con_PATH=$ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2013-1792/ConConfig.2013-1792
$ROOT_DIR/tool/staticAnalysis/DBDS-INSTRU/dbds-clang-fast++ -g -fsanitize=address ./2013-1792.cpp -o 2013-1792 -lpthread -ldl

# CVE-2015-7550
cd $ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2015-7550
./cleanDIR.sh
clang++ -g -emit-llvm -c ./2015-7550.cpp -o 2015-7550.bc
$ROOT_DIR/tool/staticAnalysis/staticAnalysis.sh 2015-7550
export Con_PATH=$ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2015-7550/ConConfig.2015-7550
$ROOT_DIR/tool/staticAnalysis/DBDS-INSTRU/dbds-clang-fast++ -g -fsanitize=address ./2015-7550.cpp -o 2015-7550 -lpthread -ldl

# CVE-2016-1972
cd $ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2016-1972
./cleanDIR.sh
clang -g -emit-llvm -c ./2016-1972.c -o 2016-1972.bc
$ROOT_DIR/tool/staticAnalysis/staticAnalysis.sh 2016-1972
export Con_PATH=$ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2016-1972/ConConfig.2016-1972
$ROOT_DIR/tool/staticAnalysis/DBDS-INSTRU/dbds-clang-fast -g -fsanitize=address -c ./2016-1972.c -o 2016-1972.o
clang++ ./2016-1972.o $ROOT_DIR/tool/staticAnalysis/DBDS-INSTRU/DBDSFunction.o -g -o 2016-1972 -lpthread -fsanitize=address -ldl

# CVE-2016-1973
cd $ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2016-1973
./cleanDIR.sh
clang++ -g -emit-llvm -c ./2016-1973.cpp -o 2016-1973.bc
$ROOT_DIR/tool/staticAnalysis/staticAnalysis.sh 2016-1973
export Con_PATH=$ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2016-1973/ConConfig.2016-1973
$ROOT_DIR/tool/staticAnalysis/DBDS-INSTRU/dbds-clang-fast++ -g -fsanitize=address ./2016-1973.cpp -o 2016-1973 -lpthread -ldl

# CVE-2016-7911
cd $ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2016-7911
./cleanDIR.sh
clang -g -emit-llvm -c ./2016-7911.cpp -o 2016-7911.bc
$ROOT_DIR/tool/staticAnalysis/staticAnalysis.sh 2016-7911
export Con_PATH=$ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2016-7911/ConConfig.2016-7911
$ROOT_DIR/tool/staticAnalysis/DBDS-INSTRU/dbds-clang-fast++ -g -fsanitize=address ./2016-7911.cpp -o 2016-7911 -lpthread -ldl

# CVE-2016-9806
cd $ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2016-9806
./cleanDIR.sh
clang -g -emit-llvm -c ./2016-9806.cpp -o 2016-9806.bc
$ROOT_DIR/tool/staticAnalysis/staticAnalysis.sh 2016-9806
export Con_PATH=$ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2016-9806/ConConfig.2016-9806
$ROOT_DIR/tool/staticAnalysis/DBDS-INSTRU/dbds-clang-fast++ -g -fsanitize=address ./2016-9806.cpp -o 2016-9806 -lpthread -ldl

# CVE-2017-15265
cd $ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265
./cleanDIR.sh
clang -g -emit-llvm -c ./2017-15265.cpp -o 2017-15265.bc
$ROOT_DIR/tool/staticAnalysis/staticAnalysis.sh 2017-15265
export Con_PATH=$ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2017-15265/ConConfig.2017-15265
$ROOT_DIR/tool/staticAnalysis/DBDS-INSTRU/dbds-clang-fast++ -g -fsanitize=address ./2017-15265.cpp -o 2017-15265 -lpthread -ldl

# CVE-2017-6346
cd $ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2017-6346
./cleanDIR.sh
clang -g -emit-llvm -c ./2017-6346.cpp -o 2017-6346.bc
$ROOT_DIR/tool/staticAnalysis/staticAnalysis.sh 2017-6346
export Con_PATH=$ROOT_DIR/evaluation/ConVul-CVE-Benchmarks/CVE-2017-6346/ConConfig.2017-6346
$ROOT_DIR/tool/staticAnalysis/DBDS-INSTRU/dbds-clang-fast++ -g -fsanitize=address ./2017-6346.cpp -o 2017-6346 -lpthread -ldl


